#include <iostream>
#include <fstream>
#include <vector>
#include <string>
#include <map>
// #include <strstream>   // old g++; deprecated
#include <sstream>
#include <iomanip>
#include <cstdio>         // to enable "remove()"
#include "errors.h"
#include "util.h"

typedef double                       dbl;
typedef const double                cdbl;
typedef const char                  cchr;
typedef const string                cstr;
typedef unsigned                     uns;
typedef const unsigned              cuns;
typedef vector<vector<double> >   matrix;
typedef istringstream           isstream;
typedef ostringstream           osstream;
typedef multimap<uns, string, greater<uns> > UnsStrMMap;

enum no_yes  {no, yes};
enum off_on  {off, on};
enum grtype  {pl, lg, wg}; // pl: p or np; lg: lettergrade; wg: withdrawn
cdbl version = 2.2;        // version number
cuns     nbpts = 9;        // 9 break points (10 categories)
cuns  maxlen = 100;        // max line length in input file

#ifdef __GLIBC__ // .pl 9999 ensures a single page
cstr nroff_cmds = ".ND\n.na\n.nr LL 70n\n.nr HY 0\n.pl 9999n\n.LP\n.hy 0\n.nf\n";
#else            // non-linux nroff has a lower page length limit
cstr nroff_cmds = ".ND\n.na\n.nr LL 70n\n.nr HY 0\n.pl 1200n\n.LP\n.hy 0\n.nf\n";
#endif
cstr MR     =  "Mr. ";     // various pronouns
cstr HE     =  "He\n";
cstr HELC   =  "he\n";
cstr HIS    = "His\n";
cstr HISLC  = "his\n";
cstr MS     =  "Ms. ";
cstr SHE    = "She\n";
cstr SHELC  = "she\n";
cstr HER    = "Her\n";
cstr HERLC  = "her\n";

cstr MID    = "mid_";      // item types
cstr FIN    =  "fin";
cstr QUIZ   = "quiz";
cstr HMWK   = "hmwk";

cstr Ap     = "A+\n";      // grade strings
cstr A      = "A\n";
cstr Am     = "A-\n";
cstr Bp     = "B+\n";
cstr B      = "B\n";
cstr Bm     = "B-\n";
cstr Cp     = "C+\n";
cstr C      = "C\n";
cstr D      = "D\n";
cstr F      = "F\n";
cstr P      = "P\n";
cstr NP     = "NP\n";
cstr W      = "W\n";

struct misc {
    no_yes hmwk;       // yes if homework is evaluated
    no_yes upgrade;    // yes if discount low-score pre-final exams
    no_yes quizzes;    // yes if quizzes are evaluated
    no_yes plus;       // yes if distinguish A+, A, B+, B grades
    no_yes rank;       // yes if want class ranking by category
    no_yes old;        // yes if we're using old style headers
    off_on debug;      // on to display details of scoring process
	off_on upgradeflag;// on to include upgrade phrase in evaluation
    uns    mids;       // number of midterms taken into account
    uns    items;      // number of items used to evaluate performance
    vector<string> tag;// labels item types
};

// prototypes

void calc_tot_bps(matrix&, vector<dbl>&, uns);
uns countwords(string&);
string eval_item(uns, dbl);
void findcats(matrix&, vector<dbl>&, vector<uns>&, uns, uns);
void ggassert(cchr*, int, cchr*, cchr*, bool);
void get_bps_and_wt(string&, matrix&, vector<dbl>&);
enum grtype get_grtype(string&);
vector<uns> get_item_indices(misc*);
string getfullname(misc*, string&);
string getname(string&);
string trimspace(string&);
string pad_space(string&, uns);
void get_opts(int, char**, misc*);
void get_template_data(misc*, matrix&, vector<dbl>&, char**);
void get_total(misc*, vector<dbl>&, vector<dbl>&);
void getweights(vector<dbl>&, misc*);
void mk_roff_file(string&);
void mk_rank_file(UnsStrMMap&, string&);
void normalize_wts(misc*, vector<dbl>&);
void prn_info(char**);
void prn_usage(char**);
void rank(misc*, UnsStrMMap&, string&, uns);
void setpronouns(string&);
void show_details(misc*, string&, matrix&, vector<dbl>&, \
        vector<dbl>&, vector<uns>&);
void showstat(map<string, uns>, string);
void togradepage(misc*, string&, ofstream&, uns);
void upgrade(misc*, matrix&, vector<dbl>&, vector<dbl>&, vector<uns>&);
map<string, uns> write_eval(misc*, ofstream&, vector<dbl>&,\
		string&, vector<uns>);
