// writes an evaluation on the output stream ofs
// it is not difficult to alter the phrases; just follow the
// patterns of the existing ones below.  note that here,
// evaluation phrases for total performance are identical
// for C and C+, and for B- and B, and for A- and A; these
// could be changed if desired so as to make them not
// identical.
//
#include "mkevals.h"

map<string,uns> write_eval(misc* mp, ofstream& ofs, vector<dbl>& item,
        string& header, vector<uns> cat)
{
    extern string mr, He, he, His, his; // generic pronouns
    static map<string, uns> ngr;        // counts grades
    grtype gt = get_grtype(header);     // gt=0: P or NP; gt=1: letter grade 
                                        // gt=2: withdrawn (W)
    vector<uns> indx = get_item_indices(mp);
    string name = getname(header);      // gets name
    setpronouns(header);                // gets sex and sets pronouns
    ofs << ".nf\n";                     // "no fill"
    ofs << ".sp\n";
    ofs << header << "\n";              // write header line on ofstream
    ofs << ".fi\n";                     // "fill"
//
// first of all, deal with withdrawn and clear failing grades:
//
    if (gt == wg){
        ngr["W"]++;
        ofs << mr << name
            << "\nwithdrew from the course.\n";
        return ngr;                 // no more added to evaluation
    }
    if (item.back() == 0.0) {       // total score = 0.0
        if (gt) ngr["F"]++;         // "F"
        else    ngr["NP"]++;        // "NP"
        ofs << mr << name
            << "\napparently did not participate in the course.\n"
            << He << "took none of the exams.\n";
        if (mp->hmwk)
            ofs << He << "also submitted no homework.\n";
        return ngr;   // no more is added to evaluation
    }
    if (item[indx[0]] == 0.0){      // final exam score = 0.0
        if (gt) ngr["F"]++;         // "F"
        else    ngr["NP"]++;        // "NP"
        ofs << mr << name
            << "\ndid not take the final exam.  Thus\n"
            << his << "overall performance is\n"
            "not sufficient to warrant a passing grade for the course.\n";
        if (mp->hmwk)
            if (cat[indx.back()]==0)// lowest category for hmwk
                ofs << "In addition,\n" << he
                    << "submitted little or no homework.\n";
        return ngr;   // no more is added to evaluation
    }

//
// now write the evaluation based on the total score:
//
    ofs << mr << name << "\'s overall performance";
    if (mp->upgradeflag){           // add extra phrase if upgraded
        ofs << ", taking into account\n" << his
            << "significant improvement on " << his << "final exam,";
        mp->upgradeflag = off;
    }
    switch (cat.back()){            // cat.back(): total score category
    case 0:
        if (gt) ngr["F"]++;
        else    ngr["NP"]++;
        ofs << " was too weak to warrant a passing grade for the course.\n";
        break;
    case 1:
        if (gt){
	    ngr["D"]++;
	    ofs << ", while showing understanding\n"
		"of a few topics, was only strong enough to\n"
		"warrant a D grade for the course.\n";
	}
        else  {
	    ngr["NP"]++;
	    ofs << ", while showing understanding\n"
		"of a few topics, was not sufficiently strong to\n"
		"warrant a passing grade for the course.\n";
	}
        if (mp->hmwk)
            if (cat[indx.back()]==0)// lowest category for hmwk
                ofs << "In addition,\n" << he
                    << "submitted little or no homework.\n";
        break;
    case 2:
        if (gt) ngr["C"]++;
        else    ngr["P"]++;
        ofs << " was adequate.\n" << He
            << "demonstrated a sufficient understanding of the topics\n"
            "presented to warrant a passing grade for the course.\n";
        break;
    case 3:
        if (gt){
            if (mp->plus) ngr["C+"]++;
            else ngr["C"]++;
        }
        else    ngr["P"]++;
        ofs << " was adequate.\n" << He
            << "demonstrated a sufficient understanding of the topics\n"
            "presented to warrant a passing grade for the course.\n";
        break;
    case 4:
        if (gt){
            if (mp->plus) ngr["B-"]++;
            else ngr["B"]++;
        }
        else    ngr["P"]++;
        ofs << " was good.\n" << He
            << "demonstrated a good grasp of the basic concepts,\n"
            "and an ability to apply the concepts to solve non-trivial\n"
            "problems.\n";
        break;
    case 5:
        if (gt) ngr["B"]++;
        else    ngr["P"]++;
        ofs << " was good.\n" << He
            << "demonstrated a good grasp of the basic concepts,\n"
            "and an ability to apply the concepts to solve non-trivial\n"
            "problems.\n";
        break;
    case 6:
        if (gt){
            if (mp->plus) ngr["B+"]++;
            else ngr["B"]++;
        }
        else    ngr["P"]++;
        ofs << " was very good.\n" << He
            << "demonstrated a clear grasp of the basic concepts,\n"
            "and an ability to apply the concepts to solve\n"
            "non-trivial problems with clear reasoning.\n";
        break;
    case 7:
        if (gt){
            if (mp->plus) ngr["A-"]++;
            else ngr["A"]++;
        }
        else    ngr["P"]++;
        ofs << " was excellent.\n" << He
            << "demonstrated a thorough grasp of the basic concepts,\n"
            "and was able to apply the concepts to solve often quite\n"
            "difficult problems.\n";
        break;
    case 8:
        if (gt) ngr["A"]++;
        else    ngr["P"]++;
        ofs << " was excellent.\n" << He
            << "demonstrated a thorough grasp of the basic concepts,\n"
            "and was able to apply the concepts to solve often quite\n"
            "difficult problems.\n";
        break;
    case 9:
        if (gt){
            if (mp->plus) ngr["A+"]++;
            else ngr["A"]++;
        }
        else    ngr["P"]++;
        ofs << " was outstanding.\n" << He
            << "demonstrated a thorough grasp of the course material,\n"
            "and was among the top few students in the class.\n";
        break;
    default:  // should not get here...
        err("did not obtain a total score category for ", name);
    }

//
// now write evaluations for each exam:
//
    ofs << ".sp\n";              // put a space first
    if (mp->mids){               // there may be midterms
        ofs << "On the basis of numerical scores, " << his
            << "performances on individual exams were as follows:\n";
        ofs << ".sp\n.nf\n&& \n";// &&: registrar's "no fill" flag

        switch (mp->mids){
        case 1:                      // just one midterm
            ofs << "Midterm exam:        "
                << eval_item(cat[indx[1]], item[indx[1]]);
            break;
        case 2:                      // two midterms
            ofs << "First midterm:       "
                << eval_item(cat[indx[1]], item[indx[1]]);
            ofs << "Second midterm:      "
                << eval_item(cat[indx[2]], item[indx[2]]);
            break;
        case 3:                      // three midterms
            ofs << "First midterm:       "
                << eval_item(cat[indx[1]], item[indx[1]]);
            ofs << "Second midterm:      "
                << eval_item(cat[indx[2]], item[indx[2]]);
            ofs << "Third midterm:       "
                << eval_item(cat[indx[3]], item[indx[3]]);
            break;
        default:
            err("you'll have to revise the code to deal with more than "
                    "three midterms");
        }
    }
    else if (mp->quizzes){           // OR there may be quizzes
        ofs << "On the basis of numerical scores, " << his
            << "performances on weekly quizzes and the final\n"
            "exam were as follows:\n";
        ofs << ".sp\n.nf\n&& \n";
        ofs << "Weekly quizzes:     "
                << eval_item(cat[indx[1]], item[indx[1]]);
    }
    ofs << "Comprehensive final: "   // ALWAYS a final exam
        << eval_item(cat[indx[0]], item[indx[0]]);
    ofs << "&&\n.fi\n";

//
// now finally write an evaluation for homework:
//
    if (mp->hmwk){
        switch (cat[indx.back()]){
        case 7:    // any of the top three categories
        case 8:
        case 9:
            ofs << ".sp\n";
            ofs << His  << "performance on the weekly homework "
                "was particularly good.\n";
            break;
        default:
            break;
        }
    }
    return ngr; // for use at the end of the main loop;
}
